import { ChangeEvent, KeyboardEvent, forwardRef } from "react";
import "./style.css";
import { eventNames } from "process";

//         interface  인풋 박스 컴포넌트 프로퍼티          //
interface Props {
  label: string;
  type: "text" | "password" /* 타입을 가져옴 */;
  placeholder: string;
  value: string;
  onChange: (event: ChangeEvent<HTMLInputElement>) => void;
  error: boolean;

  icon?: "eye-light-off-icon" | "eye-light-on-icon" | "expand-right-light-icon";
  onButtonClick?: () => void;

  message?: string;

  onKeyDown?: (event: KeyboardEvent<HTMLInputElement>) => void;
}

//             component Input Box 컴포넌트                //
const InputBox = forwardRef<HTMLInputElement, Props>((props: Props, ref) => {
  //             state: 프로퍼티들
  const { label, type, error, placeholder, value, icon, message } = props;
  const { onChange, onButtonClick, onKeyDown } = props;

  /*   //             event Handler ㅣ 입력값 변경 이벤트 처리                 //
  const onChangeHandler = (event: ChangeEvent<HTMLInputElement>) => {
    const { value } = event.target;
    setValue(value);
  }; */

  //             event Handler | 키 이벤트 처리함수             //
  const onKeyDownHandler = (event: KeyboardEvent<HTMLInputElement>) => {
    if (!onKeyDown) return;
    onKeyDown(event);
  };

  //             render : input box 컴포넌트                     //
  return (
    <div className="inputbox">
      <div className="inputbox-label">{label}</div>
      <div
        className={error ? "inputbox-container-error" : "inputbox-container"}
      >
        <input
          ref={ref}
          type={type}
          className="input"
          placeholder={placeholder}
          value={value}
          onChange={onChange}
          onKeyDown={onKeyDownHandler}
        />
        {onButtonClick !== undefined && (
          <div className="icon-button" onClick={onButtonClick}>
            {icon !== undefined && <div className={`icon ${icon}`}></div>}
          </div>
        )}
      </div>
      {message !== undefined && (
        <div className="inputbox-message">{message}</div>
      )}
    </div>
  );
});

export default InputBox;
