import { useState, ChangeEvent, useRef, KeyboardEvent, useEffect } from "react";
import "./style.css";
import { useLocation, useNavigate, useParams } from "react-router-dom";
import {
  AUTH_PATH,
  BOARD_DETAIL_PATH,
  BOARD_PATH,
  BOARD_UPDATE_PATH,
  BOARD_WRITE_PATH,
  MAIN_PATH,
  SEARCH_PATH,
  USER_PATH,
  EMPLOYEE_MANAGEMENT_PATH,
  PERMISSION_MANAGEMENT_PATH,
  NOTICE_LIST_PATH,
  NOTICE_WRITE_PATH,
} from "constant";
import { useCookies } from "react-cookie";
import { useBoardStore, useLoginuserStore } from "stores";
import { fileUploadRequest, patchBoardRequest, postBoardRequest } from "apis";
import { PatchBoardRequestDto, PostBoardRequestDto } from "apis/request/board";
import {
  PatchBoardResponseDto,
  PostBoardResponseDto,
} from "apis/response/board";
import { ResponseDto } from "apis/response";
import MenuItem from "components/MenuItem/menuItem";

//           state: 헤더 레이아웃
export default function Header() {
  //           state: 상태 정의
  const [cookies, setCookie] = useCookies();
  const { pathname } = useLocation();
  const { loginUser, setLoginUser, resetLoginUser } = useLoginuserStore();
  const [isLogin, setLogin] = useState<boolean>(false);
  const [isAuthPage, setAuthPage] = useState<boolean>(false);
  const [isMainpage, setMainpage] = useState<boolean>(false);
  const [isSearchPage, setSearchPage] = useState<boolean>(false);
  const [isBoardDetailPage, setBoardDetailPage] = useState<boolean>(false);
  const [isBoardWritePage, setBoardWritePage] = useState<boolean>(false);
  const [isNoticeWritePage, setNoticeWritePage] = useState<boolean>(false);
  const [isBoardUPdatePage, setBoardUPdatePage] = useState<boolean>(false);
  const [isUserPage, setUserPage] = useState<boolean>(false);
  const [isEmployeePage, setEmployeePage] = useState<boolean>(false);
  const navigate = useNavigate();

  //           function: 네비게이트 함수          //
  const onLogClickHandler = () => {
    navigate(MAIN_PATH());
  };

  //           event handler: 메뉴 클릭 핸들러          //
  const onMenuClickHandler = (path: string) => {
    navigate(path);
  };

  //          event handler: 사이드 카드 클릭 이벤트 처리           //
  const onSideCardClickHandler = () => {
    navigate(NOTICE_WRITE_PATH());
  };

  const onMenuEmployeeClickHandler = () => {
    navigate(EMPLOYEE_MANAGEMENT_PATH());
  };

  //           component: 검색 버튼 컴포넌트          //
  const SearchButton = () => {
    const searchButtonRef = useRef<HTMLDivElement | null>(null);
    const [status, setStatus] = useState<boolean>(false);
    const [word, setWord] = useState<string>("");
    const { searchWord } = useParams();

    const onSearchWordChangeHandler = (
      event: ChangeEvent<HTMLInputElement>
    ) => {
      setWord(event.target.value);
    };

    const onSearchWordkeyDownHandler = (
      event: KeyboardEvent<HTMLInputElement>
    ) => {
      if (event.key !== "Enter") return;
      if (!searchButtonRef.current) return;
      searchButtonRef.current.click();
    };

    const onSearchButtonClickHandler = () => {
      if (!status) {
        setStatus(!status);
        return;
      }
      navigate(SEARCH_PATH(word));
    };

    useEffect(() => {
      if (searchWord) {
        setWord(searchWord);
        setStatus(true);
      }
    }, [searchWord]);

    if (!status)
      return (
        <div className="icon-button" onClick={onSearchButtonClickHandler}>
          <div className="icon search-light-icon"></div>
        </div>
      );
    return (
      <div className="header-search-input-box">
        <input
          className="header-search-input"
          type="text"
          placeholder="검색어를 입력해주세요."
          value={word}
          onChange={onSearchWordChangeHandler}
          onKeyDown={onSearchWordkeyDownHandler}
        />
        <div
          ref={searchButtonRef}
          className="icon-button"
          onClick={onSearchButtonClickHandler}
        >
          <div className="icon search-light-icon"></div>
        </div>
      </div>
    );
  };

  //           component: 마이페이지 버튼 컴포넌트          //
  const MyPageButton = () => {
    const { userEmail } = useParams();

    const onMyPageButtonClickHandler = () => {
      if (!loginUser) return;
      const { email } = loginUser;
      navigate(USER_PATH(email));
    };

    const onSignOutButtonClickHandler = () => {
      resetLoginUser();
      setCookie("accessToken", "", { path: MAIN_PATH(), expires: new Date() });
      navigate(MAIN_PATH());
    };

    const onSignInButtonClickHandler = () => {
      navigate(AUTH_PATH());
    };

    if (isLogin && userEmail === loginUser?.email)
      return (
        <div className="white-button" onClick={onSignOutButtonClickHandler}>
          {"로그아웃"}
        </div>
      );
    if (isLogin)
      return (
        <div className="white-button" onClick={onMyPageButtonClickHandler}>
          {"마이페이지"}
        </div>
      );

    return (
      <div className="black-button" onClick={onSignInButtonClickHandler}>
        {"로그인"}
      </div>
    );
  };

  //           component: 업로드 버튼 컴포넌트          //
  const UploadButton = () => {
    const { boardNumber } = useParams();
    const { title, content, boardImageFileList, resetBoard } = useBoardStore();

    const postBoardResponse = (
      responseBody: PostBoardResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "AF" || code === "NU") navigate(AUTH_PATH());
      if (code === "DBE") alert("데이터 베이스 오류입니다.");
      if (code === "VF") alert("제목과 내용은 필수입니다.");
      if (code !== "SU") return;

      resetBoard();
      if (!loginUser) return;
      const { email } = loginUser;
      navigate(USER_PATH(email));
    };

    const patchBoardResponse = (
      responseBody: PatchBoardResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "AF" || code === "NU" || code === "NB" || code === "NP")
        navigate(AUTH_PATH());
      if (code === "DBE") alert("데이터 베이스 오류입니다.");
      if (code === "VF") alert("제목과 내용은 필수입니다.");
      if (code !== "SU") return;

      if (!boardNumber) return;
      if (isBoardWritePage) {
        navigate(BOARD_PATH() + "/" + BOARD_DETAIL_PATH(boardNumber));
        alert("작성 완료되었습니다.");
      } else if (isNoticeWritePage) {
        MAIN_PATH();
        alert("작성 완료되었습니다.");
      }
    };

    const onUploadButtonClickHandler = async () => {
      const accessToken = cookies.accessToken;
      if (!accessToken) return;

      const boardImageList: string[] = [];

      for (const file of boardImageFileList) {
        const data = new FormData();
        data.append("file", file);

        const url = await fileUploadRequest(data);
        if (url) boardImageList.push(url);
      }

      const isWriterPage = pathname === BOARD_PATH() + "/" + BOARD_WRITE_PATH();
      const isNoticeWritePage = pathname === NOTICE_WRITE_PATH();
      if (isWriterPage || isNoticeWritePage) {
        const requestBody: PostBoardRequestDto = {
          title,
          content,
          boardImageList,
        };
        postBoardRequest(requestBody, accessToken).then(postBoardResponse);
      } else {
        if (!boardNumber) return;
        const requestBody: PatchBoardRequestDto = {
          title,
          content,
          boardImageList,
        };
        patchBoardRequest(boardNumber, requestBody, accessToken).then(
          patchBoardResponse
        );
      }

      const requestBody: PostBoardRequestDto = {
        title,
        content,
        boardImageList,
      };
      postBoardRequest(requestBody, accessToken).then(postBoardResponse);
    };

    if (title && content)
      return (
        <div className="black-button" onClick={onUploadButtonClickHandler}>
          {"업로드"}
        </div>
      );
    return <div className="disable-button">{"업로드 불가"}</div>;
  };

  useEffect(() => {
    const isAuthPage = pathname.startsWith(AUTH_PATH());
    setAuthPage(isAuthPage);

    const isMainpage = pathname === MAIN_PATH();
    setMainpage(isMainpage);

    const isSearchPage = pathname.startsWith(SEARCH_PATH(""));
    setSearchPage(isSearchPage);

    const isBoardDetailPage = pathname.startsWith(
      BOARD_PATH() + "/" + BOARD_DETAIL_PATH("")
    );
    setBoardDetailPage(isBoardDetailPage);

    const isBoardWritePage = pathname.startsWith(
      BOARD_PATH() + "/" + BOARD_WRITE_PATH()
    );
    setBoardWritePage(isBoardWritePage);

    const isNoticeWritePage = pathname.startsWith(NOTICE_WRITE_PATH());
    setNoticeWritePage(isNoticeWritePage);

    const isBoardUPdatePage = pathname.startsWith(
      BOARD_PATH() + "/" + BOARD_UPDATE_PATH("")
    );
    setBoardUPdatePage(isBoardUPdatePage);

    const isUserPage = pathname.startsWith(USER_PATH(""));
    setUserPage(isUserPage);

    const isEmployeePage = pathname.startsWith(EMPLOYEE_MANAGEMENT_PATH());
    setEmployeePage(isEmployeePage);
  }, [pathname]);

  useEffect(() => {
    setLogin(loginUser !== null);
  }, [loginUser]);

  //           state: 서브 메뉴바 상태               //
  const [showSubMenu, setShowSubMenu] = useState(null);

  const handleMouseEnter = (menu: any) => {
    setShowSubMenu(menu);
  };

  const handleMouseLeave = () => {
    setShowSubMenu(null);
  };
  //          render: 헤더 렌더링          //
  return (
    <div id="header">
      <div className="header-container">
        <div className="header-left-box" onClick={onLogClickHandler}>
          <div className="icon-box">
            <div className="icon logo-dark-icon"></div>
          </div>
          <div className="header-logo">{"Layouts Header"}</div>
        </div>
        {isLogin && (
          <div className="header-menu-box">
            <div
              className="header-menu"
              onMouseEnter={() => handleMouseEnter("employee")}
              onMouseLeave={handleMouseLeave}
            >
              <MenuItem
                text="사원 관리"
                imageSrc="https://cdn-icons-png.flaticon.com/512/159/159833.png"
                onClick={() => onMenuClickHandler(EMPLOYEE_MANAGEMENT_PATH())}
              />
              {showSubMenu === "employee" && (
                <div className="sub-menu">
                  <div onClick={() => onMenuEmployeeClickHandler()}>
                    사원 추가
                  </div>
                  <div>사원 목록</div>
                </div>
              )}
            </div>
            <div
              className="header-menu"
              onMouseEnter={() => handleMouseEnter("permission")}
              onMouseLeave={handleMouseLeave}
            >
              <MenuItem
                text="권한 관리"
                imageSrc="https://cdn.iconscout.com/icon/premium/png-256-thumb/access-granted-3192235-2660379.png?f=webp"
                onClick={() => onMenuClickHandler(PERMISSION_MANAGEMENT_PATH())}
              />
              {showSubMenu === "permission" && (
                <div className="sub-menu">
                  <div>권한 추가</div>
                  <div>권한 목록</div>
                </div>
              )}
            </div>
            <div
              className="header-menu"
              onMouseEnter={() => handleMouseEnter("notice")}
              onMouseLeave={handleMouseLeave}
            >
              <MenuItem
                text="공지 사항"
                imageSrc="https://png.pngtree.com/png-vector/20190118/ourmid/pngtree-vector-announcement-icon-png-image_323832.jpg"
                onClick={() => onMenuClickHandler(NOTICE_LIST_PATH())}
              />
              {showSubMenu === "notice" && (
                <div className="sub-menu">
                  <div onClick={() => onMenuClickHandler(NOTICE_WRITE_PATH())}>
                    공지 추가
                  </div>
                  <div onClick={() => onMenuClickHandler(NOTICE_LIST_PATH())}>
                    공지 목록
                  </div>
                </div>
              )}
            </div>
          </div>
        )}

        <div className="header-right-box">
          {(isAuthPage || isMainpage || isSearchPage || isBoardDetailPage) && (
            <SearchButton />
          )}
          {(isMainpage ||
            isSearchPage ||
            isBoardDetailPage ||
            isUserPage ||
            isEmployeePage) && <MyPageButton />}
          {(isBoardWritePage || isBoardUPdatePage || isNoticeWritePage) && (
            <UploadButton />
          )}
        </div>
      </div>
    </div>
  );
}
