import React, { useEffect, useState } from "react";
import "./style.css";
import Top3Item from "components/Top3Item";
import { BoardListItem } from "types/interface";
import BoardItem from "components/BoardItem";
import Pagination from "components/Pagination";
import { useNavigate, useParams } from "react-router-dom";
import { MAIN_PATH, SEARCH_PATH } from "constant";
import {
  getLatestBoardListRequset,
  getPopularListRequest,
  getTop3BoardListRequest,
  getUserRequest,
} from "apis";
import {
  GetLatestBoardListResponseDto,
  GetTop3BoardListResponseDto,
} from "apis/response/board";
import { ResponseDto } from "apis/response";
import { usePagination } from "hooks";
import { GetPoplarListResponseDto } from "apis/response/search";
import { useCookies } from "react-cookie";
import { GetUserResponseDto } from "apis/response/user";
import { useLoginuserStore } from "stores";

import DefaultProfileImage from "assets/image/default-profile-image.png";

//          component 메인 화면 컴포넌트           //
export default function Main() {
  //          function: 네비게이트 함수           //
  const navigate = useNavigate();

  //          component 메인 화면 상단 컴포넌트           //
  const MainTop = () => {
    //          state: 주간 top3 게시물 리스트 상태            //
    const [top3BoardList, setTop3BoardList] = useState<BoardListItem[]>([]);

    //          state: 로그인 상태            //
    const [isLogin, setLogin] = useState<boolean>(false);

    //          state: 로그인 유저 상태            //
    const { loginUser } = useLoginuserStore();

    //          state: 쿠키 상태             //
    const [cookies, setCookies] = useCookies();

    //          state: userEamil path variable 여부 상태            //
    const { userEmail } = useParams();

    //          state: 닉네임 상태           //
    const [nickname, setNickname] = useState<string>("");

    //          state: 프로필 이미지 상태           //
    const [profileImage, setProfileImage] = useState<string | null>(null);

    //          function: getTop3BoardListResponse 처리 함수            //
    const getTop3BoardListResponse = (
      responseBody: GetTop3BoardListResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "DBE") alert("데이터베이스 오류입니다.");
      if (code !== "SU") return;

      const { top3List } = responseBody as GetTop3BoardListResponseDto;
      setTop3BoardList(top3List);
    };

    //          function: getUserResponse 처리 함수            //
    const getUserResponse = (
      responseBody: GetUserResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "NU") alert("존재하지 않는 유저입니다.");
      if (code === "DBE") alert("데이터베이스 오류입니다");
      if (code !== "SU") {
        navigate(MAIN_PATH());
        return;
      }
      const { email, nickname, profileImage } =
        responseBody as GetUserResponseDto;
      setNickname(nickname);
      setProfileImage(profileImage);
      /* const isMyPage = email === loginUser?.email;
      setMyPage(isMyPage); */
    };

    //          effect: 첫 마운트 시 실행될 함수            //
    useEffect(() => {
      getTop3BoardListRequest().then(getTop3BoardListResponse);
    }, []);

    //          effect: accessToken cookie 값이 변경될 때 마다 실행할 함수--------------           //
    useEffect(() => {
      const accessToken = cookies.accessToken;
      if (accessToken !== null) {
        setLogin(true);
      } else {
        setLogin(false); // accessToken이 없으면 로그인 상태를 false로 설정
      }
    }, [cookies]);

    //          effect: user email path variable 변경시 실행 할 함수            //
    useEffect(() => {
      if (!loginUser) return; // loginUser가 없으면 함수 실행 중단
      const { email } = loginUser; // loginUser에서 email 추출
      getUserRequest(email).then(getUserResponse);
      console.log(email);
    }, [loginUser]); // loginUser가 변경될 때마다 실행

    //          render 메인화면 상단 컴포넌트 렌더링           //
    return (
      <div id="main-top-wrapper">
        <div className="main-top-container">
          {loginUser && (
            <div className="main-work-wrapper">
              <div className="main-top-contents-box1">
                <div className="main-top-contents-title">{"주간 게시글"}</div>
                <div className="main-top-contents">
                  {top3BoardList.slice(0, 2).map((top3ListItem) => (
                    <Top3Item top3ListItem={top3ListItem} />
                  ))}
                </div>
              </div>
              <div className="main-work-container">
                <div
                  className="profile-box"
                  style={{
                    backgroundImage: `url(${
                      profileImage ? profileImage : DefaultProfileImage
                    })`,
                  }}
                ></div>

                <div className="welcome">
                  <span style={{ fontWeight: "bold" }}>{nickname}</span>님
                  환영합니다.
                </div>
                <div className="button-box">
                  <div className="work">{"출근"}</div>
                  <div className="work-out">{"퇴근"}</div>
                  <div className="vacation">{"휴가"}</div>
                  <div className="work-other-place">{"출장"}</div>
                </div>
              </div>
            </div>
          )}

          {!loginUser && (
            <>
              <div className="main-top-title">
                {"Project Demo\n-Main Page-"}
              </div>
              <div className="main-top-contents-box">
                <div className="main-top-contents-title">{"주간 게시글"}</div>
                <div className="main-top-contents">
                  {top3BoardList.map((top3ListItem) => (
                    <Top3Item top3ListItem={top3ListItem} />
                  ))}
                </div>
              </div>
            </>
          )}
        </div>
      </div>
    );
  };
  //          component 메인 화면 하단 컴포넌트           //
  const MainBottom = () => {
    //           state: 페이지네이션 관련 상태           //
    const {
      currentPage /* 현재 페이지가 어떤 위치에 있는지 */,
      setCurrentPage,
      currentSection,
      setCurrentSection,
      viewList /* 현재 보여줄 리스트 */,
      viewPageList,
      totalSection /* 전체 섹션이 몇개인지 */,
      setTotalList,
    } = usePagination<BoardListItem>(3); /* 5개씩 게시물 보여짐 */
    //           state: 인기 검색어 리스트 상태           //
    const [popularWordList, setPopularWordList] = useState<string[]>([]);

    //          function: getLatestBoardListResponse 처리함수            //
    const getLatestBoardListResponse = (
      responseBody: GetLatestBoardListResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "DBE") alert("데이터베이스 오류입니다.");
      if (code !== "SU") return;

      const { latestList } = responseBody as GetLatestBoardListResponseDto;
      setTotalList(latestList);
    };

    //          function: getPopularListResponse 처리함수             //
    const getPopularListResponse = (
      responseBody: GetPoplarListResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "DBE") alert("데이터베이스 오류입니다.");
      if (code !== "SU") return;

      const { popularWordList } = responseBody as GetPoplarListResponseDto;
      setPopularWordList(popularWordList);
    };

    //          event handler: 인기 검색어 클릭 이벤트 처리             //
    const onPopularWordClickHandler = (word: string) => {
      navigate(SEARCH_PATH(word));
    };

    //          effect: 첫 마운트 시 실행될 함수            //
    useEffect(() => {
      getLatestBoardListRequset().then(getLatestBoardListResponse);
      getPopularListRequest().then(getPopularListResponse);
    }, []);

    //          render: 메인 화면 하단 컴포넌트 렌더링         //
    return (
      <div id="main-bottom-wrapper">
        <div className="main-bottom-container">
          <div className="main-bottom-title">{"최근 공지"}</div>
          <div className="main-bottom-contents-box">
            <div className="main-bottom-current-contents">
              {viewList.map((boardListItem) => (
                <BoardItem boardListItem={boardListItem} />
              ))}
            </div>
            <div className="main-bottom-popular-box">
              <div className="main-bottom-popular-card">
                <div className="main-bottom-popular-card-box">
                  <div className="main-bottom-popular-card-title">
                    {"인기 검색어"}
                  </div>
                  <div className="main-bottom-popular-card-contents">
                    {popularWordList.map((word) => (
                      <div
                        className="word-badge"
                        onClick={() => onPopularWordClickHandler(word)}
                      >
                        {word}
                      </div>
                    ))}
                  </div>
                </div>
              </div>
            </div>
          </div>
          <div className="main-bottom-pagination-box">
            <Pagination
              currentPage={currentPage}
              currentSection={currentSection}
              setCurrentPage={setCurrentPage}
              setCurrentSection={setCurrentSection}
              viewPageList={viewPageList}
              totalSection={totalSection}
            />
          </div>
        </div>
      </div>
    );
  };
  //          render 메인화면 컴포넌트 렌더링           //
  return (
    <>
      <MainTop />
      <MainBottom />
    </>
  );
}
